package net.lecnam.info;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.logging.Logger;
import net.lecnam.info.Answer.*;
import net.lecnam.info.Request.*;

public class Server<K, V> implements Runnable {

	BlockingQueue<Request<K, V>> in;
	BlockingQueue<Answer<V>> out;
	Map<K, V> m;

	static final Logger logger = Logger.getLogger("server");

	Server(BlockingQueue<Request<K, V>> in, BlockingQueue<Answer<V>> out, Map<K, V> m) {
		this.in = in;
		this.out = out;
		this.m = new HashMap<>(m);
	}

	@Override
	public void run() {
		try {
			boolean done = false;
			while (!done) {
				Request<K, V> request = in.take();
				logger.info("Request: " + request);
				switch (request) {
					case Stop() -> {
						done = true;
					}
					case Get(var key) -> {
						V value = m.get(key);
						Answer<V> answer = new Value<>(value);
						out.put(answer);
					}
					case Put(var key, var value) -> {
						V old = m.put(key, value);
						Answer<V> answer = new OldValue<>(old);
						out.put(answer);
					}
				}
			}
		} catch (InterruptedException ex) {
			throw new Error(ex);
		}
	}

}
